<?php

declare(strict_types=1);

namespace Drupal\mautic_block\Plugin\Block;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\mautic_api\MauticApiConnector;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Provides a mautic form block.
 */
#[Block(
  id: 'mautic_block_mautic_form',
  admin_label: new TranslatableMarkup('Mautic Form'),
  category: new TranslatableMarkup('Mautic'),
)]
class MauticFormBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected MauticApiConnector $mauticApiConnector,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected RendererInterface $renderer,
    protected CacheBackendInterface $cache,
    protected TimeInterface $time,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('mautic_api.connector'),
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('cache.default'),
      $container->get('datetime.time'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $connections = $this->entityTypeManager->getStorage('mautic_api_connection')->loadMultiple();
    $options = [];
    foreach ($connections as $connection) {
      $options[$connection->id()] = $connection->label();
    }

    $form['mautic_connection'] = [
      '#type' => 'select',
      '#title' => $this->t('Mautic API Connection'),
      '#description' => $this->t('Select the Mautic API connection to use for this block.'),
      '#options' => $options,
      '#default_value' => $this->configuration['mautic_connection'] ?? NULL,
      '#required' => TRUE,
      '#empty_option' => $this->t('- Select a Connection -'),
      '#ajax' => [
        'callback' => [$this, 'getMauticFormList'],
        'wrapper' => 'mautic-form-block-wrapper',
        'event' => 'change',
      ],
    ];

    $form['mautic_form_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Select the Mautic form'),
      '#options' => [],
      '#prefix' => '<div id="mautic-form-block-wrapper">',
      '#suffix' => '</div>',
      '#validated' => TRUE,
      '#empty_option' => $this->t('- Choose a connection to retrieve forms -'),
    ];

    // If we have a connection selected, load the forms.
    if (!empty($this->configuration['mautic_connection'])) {
      $form_options = $this->getMauticForms($this->configuration['mautic_connection']);
      $form['mautic_form_id']['#options'] = $form_options;

      // Only set default value if it exists in the options.
      if (!empty($this->configuration['mautic_form_id'])) {
        $form_id = (string) $this->configuration['mautic_form_id'];
        if (isset($form_options[$form_id])) {
          $form['mautic_form_id']['#default_value'] = $form_id;
        }
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['mautic_connection'] = $form_state->getValue('mautic_connection');
    $this->configuration['mautic_form_id'] = $form_state->getValue('mautic_form_id');
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $build = [];

    // Check if we have the required configuration.
    if (empty($this->configuration['mautic_connection']) || empty($this->configuration['mautic_form_id'])) {
      return [
        '#markup' => $this->t('Mautic form is not configured. Please configure the block settings.'),
      ];
    }

    try {
      /** @var \Drupal\mautic_api\Entity\MauticApiConnection $connection */
      $connection = $this->entityTypeManager->getStorage('mautic_api_connection')
        ->load($this->configuration['mautic_connection']);

      if (!$connection) {
        throw new \Exception('Mautic connection not found.');
      }

      $base_url = $this->mauticApiConnector->getCompleteUrl($connection);
      $form_id = $this->configuration['mautic_form_id'];

      // Create the script tag for the Mautic form.
      $script = sprintf(
        '<script type="text/javascript" src="%s/form/generate.js?id=%s"></script>',
        $base_url,
        $form_id
      );

      $build['content'] = [
        '#type' => 'markup',
        '#markup' => $script,
        '#allowed_tags' => ['script'],
      ];
    }
    catch (\Exception $e) {
      $build['content'] = [
        '#markup' => $this->t('Error loading Mautic form: @error', ['@error' => $e->getMessage()]),
      ];
    }

    return $build;
  }

  /**
   * Get mautic form list.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Return the Ajax response for updating the Mautic form with lists.
   *
   * @throws \Exception
   */
  public function getMauticFormList(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $connection_id = $form_state->getValue(['settings', 'mautic_connection']);

    if ($connection_id) {
      $form_options = $this->getMauticForms($connection_id);
      $form['settings']['mautic_form_id']['#options'] = $form_options;
      $form['settings']['mautic_form_id']['#default_value'] = NULL;
      $form['settings']['mautic_form_id']['#validated'] = TRUE;

      $response->addCommand(new ReplaceCommand('#mautic-form-block-wrapper', $this->renderer->render($form['settings']['mautic_form_id'])));
    }

    return $response;
  }

  /**
   * Get Mautic forms for a connection with caching.
   *
   * @param string $connection_id
   *   The Mautic connection ID.
   *
   * @return array
   *   Array of form options.
   */
  protected function getMauticForms(string $connection_id): array {
    $cid = 'mautic_forms:' . $connection_id;

    // Try to get from cache first.
    if ($cache = $this->cache->get($cid) && isset($cache->data)) {
      return $cache->data;
    }

    // If not in cache, fetch from Mautic.
    $formData = $this->mauticApiConnector->request($connection_id, 'forms', NULL, 'list');
    $options = [];

    if (!empty($formData['forms'])) {
      foreach ($formData['forms'] as $data) {
        $options[(string) $data['id']] = $data['name'];
      }
    }

    // Cache the results for 1 hour.
    $this->cache->set($cid, $options, $this->time->getRequestTime() + 3600);

    return $options;
  }

}
